#include	<sys/file.h>
#include	<sys/types.h>
#include	<time.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<stdio.h>

#include	<db.h>

#include	"nbs.h"

extern	int	errno;

/*
	Marcus J. Ranum, 2004   - All rights reserved
	$Id: nbsdump.c,v 1.1 2004/09/03 01:16:19 mjr Exp $

	This software may be used and redistributed free of charge,
	but may not be incorporated into a commercial product or
	offering without the author's permission. Please see
	the included "LICENSE.pdf" for the software's governing
	license and terms.

	NBS - "Never Before Seen" anomaly detector database dumper
*/


static	void
usage()
{
	fprintf(stderr,"usage: dumpnbs [options]\n");
	fprintf(stderr,"\t[-d database] specify database\n");
	fprintf(stderr,"\t[-k key] specify single key to find\n");
	fprintf(stderr,"\t[-c #] number of keys to dump (default is all)\n");
	fprintf(stderr,"\t[-K -U -C] one of these (default is -K)\n");
	fprintf(stderr,"\t\t-K - dump items in alphabetic order\n");
	fprintf(stderr,"\t\t-C - dump in count frequency\n");
	fprintf(stderr,"\t\t-U - dump in order of update time\n");
	fprintf(stderr,"\t[-R] reverse sort order (last to first)\n");
	fprintf(stderr,"\t[-V] verbose output\n");
	fprintf(stderr,"\t[-M] minimalist output\n");
	fprintf(stderr,"\t[-X] pseudo-XML output\n");
	fprintf(stderr,"\t[-Y] year/date output\n");
	fprintf(stderr,"\t[-x] \"technical\" output\n");
	fprintf(stderr,"\t[-s] include database statistics\n");
	fprintf(stderr,"\t[-S] output database statistics only\n");
	fprintf(stderr,"\t[-D] debug\n");
	exit(1);
}

static	void	splatkey(NBS_DB *,char *);
static	void	dbstats(NBS_DB *);
static	void	printrec(char *,NBS_REC *);
static	int	tdecode2(char *);

static	int	dflg = 0;		/* debug flag */
static	int	rflg = 0;		/* reverse order */
static	int	vflg = 0;		/* verbosity */
static	int	yflg = 0;		/* year/date */
static	int	xflg = 0;		/* XMLish output */
static	int	mflg = 0;		/* minimalism */
static	int	aflg = 0;		/* attr=value output */
static	int	sflg = 0;		/* add db statistics */
static	int	Sflg = 0;		/* only db statistics */
static	int	kcount = 0;		/* how many to dump */
static	char	*dbfilename = NBS_DFLTFILE;
static	char	*specifickey = (char *)0;
static	int	what = 0;		/* what to dump */
static	int	masktim = -1;		/* time mask (seconds) */



int
main(int ac, char *av[])
{
	int	x;
	NBS_DB	*d;
	int	count;

	while ((x = getopt(ac, av, "CxMKUVYDRXsSd:k:c:T:")) != -1)
		switch(x) {
		case 'd':
			dbfilename = optarg;
			break;
		case 'k':
			specifickey = optarg;
			break;
		case 'c':
			kcount = atoi(optarg);
			if(kcount <= 0) {
				fprintf(stderr,"\"%s\" must be a positive integer\n",optarg);
				usage();
			}
			break;
		case 'D':
			dflg = 1;
			fprintf(stderr,"Debugging turned on\n");
			break;
		case 'C':
			if(what != 0) {
				fprintf(stderr,"cannot specify -C and other ordering value\n");
				usage();
			}
			what = 1;
			break;
		case 'K':
			if(what != 0) {
				fprintf(stderr,"cannot specify -K and other ordering value\n");
				usage();
			}
			what = 0;
			break;
		case 'U':
			if(what != 0) {
				fprintf(stderr,"cannot specify -U and other ordering value\n");
				usage();
			}
			what = 2;
			break;
		case 'M':
			mflg = 1;
			break;
		case 'x':
			aflg = 1;
			break;
		case 'R':
			rflg = 1;
			break;
		case 'V':
			vflg = 1;
			break;
		case 'X':
			xflg = 1;
			break;
		case 'Y':
			yflg = 1;
			break;
		case 's':
			sflg = 1;
			break;
		case 'S':
			Sflg = 1;
			break;
		case 'T':
			masktim = tdecode2(optarg);
			if(masktim <= 0) {
				fprintf(stderr,"cannot decode \"%s\" as a time interval\n",optarg);
				usage();
			}
			break;
		default:
			fprintf(stderr,"bad argument: -%c\n",x);
			usage();
		}

	if((d = nbs_dbopen(dbfilename)) == (NBS_DB *)0) {
		fprintf(stderr,"failed to open database: %s\n",nbs_dberror());
		exit(-1);
	}

	if(Sflg) {
		dbstats(d);
		goto done;
	}

	if(specifickey != (char *)0) {
		splatkey(d,specifickey);
		goto done;
	}

	if(what == 0)
		count = nbs_dbwalkprim(d,rflg,kcount,masktim,printrec);
	if(what == 1)
		count = nbs_dbwalkcnt(d,rflg,kcount,masktim,printrec);
	if(what == 2)
		count = nbs_dbwalkupd(d,rflg,kcount,masktim,printrec);

done:
	if(sflg)
		dbstats(d);

	if(nbs_dbclose(d)) {
		fprintf(stderr,"failed to close database: %s\n",nbs_dberror());
		exit(-1);
	}
	/* e finito! */
	exit(0);
}



/* dump a record */
static	void
printrec(char *s,NBS_REC *rp)
{
	char	*p;

	if(mflg) {
		printf("%d\t%s\n",rp->cnt,s);
		return;
	}
	if(yflg) {
		p = ctime(&rp->up);
		*(p + 24) = '\0';
		printf("%s\t%s\n",p,s);
		return;
	}
	if(aflg) {
		printf("key=%s\n",s);
		printf("count=%d\n",rp->cnt);
		printf("last update=%s",ctime(&rp->up));
		printf("first seen=%s",ctime(&rp->frst));
		if(rp->csiz != 0)
			printf("first example=%s\n",rp->cmt);
		return;
	}
	if(xflg) {
		p = ctime(&rp->up);
		*(p + 24) = '\0';
		printf("<rec>\n<key>%s</key>\n",s);
		printf("<cnt>%d</cnt>\n",rp->cnt);
		printf("<last>%s</last>\n",p);
		p = ctime(&rp->frst);
		*(p + 24) = '\0';
		printf("<first>%s</first>\n",p);
		if(rp->csiz != 0)
			printf("<example>%s</example>\n",rp->cmt);
		printf("</rec>\n");
		return;
	}
	if(vflg) {
		printf("\n%s\n",s);
		printf("seen %d times\n",rp->cnt);
		printf("last seen %s",ctime(&rp->up));
		printf("first seen %s",ctime(&rp->frst));
		if(rp->csiz != 0)
			printf("first example %s\n",rp->cmt);
		return;
	}

	/* default - base on collating order */
	if(what == 0) {
		printf("%s\tseen %d times\n",s,rp->cnt);
		return;
	}
	if(what == 1) {
		printf("%d\t%s\n",rp->cnt,s);
		return;
	}
	if(what == 2) {
		p = ctime(&rp->up);
		*(p + 24) = '\0';
		printf("%s\t%s\n",p,s);
		return;
	}
}



/* print a specific key */
static	void
splatkey(NBS_DB *d,char *s)
{
	NBS_REC	*rp;
	off_t	rec;
	int	x;
	int	len;

	len = strlen(s) + 1;
	if((rec = nbs_dbck(d,s,len,&x)) > 0) {
		if((rp = nbs_dbgetrec(d,rec,x)) == (NBS_REC *)0) {
			fprintf(stderr,"record %d get failed! %s\n",(int)rec,nbs_dberror());
			return;
		}
		printrec(s,rp);
	} else {
		if(vflg)
			printf("%s: key not found\n",s);
	}
}


static	void
dbstats(NBS_DB *d)
{
	if(aflg) {
		printf("database entries=%d\n",d->ecnt);
		printf("total items seen=%d\n",d->tlin);
		return;
	}
	printf("total database entries: %d\n",d->ecnt);
	printf("total items seen: %d\n",d->tlin);
}




static	int
tdecode2(char *tp)
{
	int	d;
	int	h;
	int	m;
	int	s;

	if(tp == (char *)0)
		return(0);
	if(sscanf(tp,"%d:%d:%d:%d",&d,&h,&m,&s) == 4)
		return((d * 60 * 60 * 24) + (h * 60 * 60) + (m * 60) + s);
	if(sscanf(tp,"%d:%d:%d",&d,&h,&m) == 3)
		return((d * 60 * 60 * 24) + (h * 60 * 60) + (m * 60));
	if(sscanf(tp,"%d:%d",&d,&h) == 2)
		return((d * 60 * 60 * 24) + (h * 60 * 60));
	if(sscanf(tp,"%d",&d) == 1)
		return((d * 60 * 60 * 24));
	return(0);
}
