#include	<sys/file.h>
#include	<sys/types.h>
#include	<time.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<string.h>
#include	<stdio.h>

#include	<db.h>

#include	"nbs.h"

extern	int	errno;

/*
	Marcus J. Ranum, 2004   - All rights reserved
	$Id: nbspreen.c,v 1.1 2004/09/03 01:16:19 mjr Exp $

	This software may be used and redistributed free of charge,
	but may not be incorporated into a commercial product or
	offering without the author's permission. Please see
	the included "LICENSE.pdf" for the software's governing
	license and terms.

	NBS - "Never Before Seen" anomaly detector database scrubber
*/


static	void
usage()
{
	fprintf(stderr,"usage: nbspreen -T d:h:m [options]\n");
	fprintf(stderr,"\t[-T d:h:m] time spec (days, hours, minutes)\n");
	fprintf(stderr,"\t\t examples 365 = one year, 0:24 = one day, 1 = one day\n");
	fprintf(stderr,"\t\t 0:12:20 = twelve hours and twenty minutes\n");
	fprintf(stderr,"\t[-d database] specify database\n");
	fprintf(stderr,"\t[-b] save old files as .BAK\n");
	fprintf(stderr,"\t[-v] verbosity\n");
	fprintf(stderr,"\t[-D] debug\n");
	exit(1);
}

static	int	tdecode(char *);
static	void	copyrec(char *,NBS_REC *);


static	int	dflg = 0;		/* debug flag */
static	int	vflg = 0;		/* verbosity flag */
static	int	bflg = 0;		/* save backups */
static	char	*dbfilename = NBS_DFLTFILE;
static	int	tpreen = (time_t)0;
static	time_t	now;
static	NBS_DB	*db_old;
static	NBS_DB	*db_new;
static	int	discards = 0;
static	int	retains = 0;


int
main(int ac, char *av[])
{
	int	x;
	int	dbflags;
	char	*tc = (char *)0;
	char	tnam[BUFSIZ];
	int	count;

	while ((x = getopt(ac, av, "T:DdVvb")) != -1)
		switch(x) {
		case 'T':
			tc = optarg;
			break;
		case 'd':
			dbfilename = optarg;
			break;
		case 'D':
			dflg = 1;
			fprintf(stderr,"Debugging turned on\n");
			break;
		case 'V':
		case 'v':
			vflg = 1;
			break;
		case 'b':
			bflg = 1;
			break;
		default:
			fprintf(stderr,"bad argument: -%c\n",x);
			usage();
		}


	tpreen = tdecode(tc);
	if(dflg)
		fprintf(stderr,"time interval is %d seconds\n",tpreen);
	if(tpreen <= (time_t)0) {
		fprintf(stderr,"Must provide purge time with -T #:#:#");
		usage();
	}

	time(&now);
	if((db_old = nbs_dbopen(dbfilename)) == (NBS_DB *)0) {
		fprintf(stderr,"failed to open database: %s\n",nbs_dberror());
		exit(-1);
	}

	strcpy(tnam,dbfilename);
	strcat(tnam,".tmp");
	dbflags = db_old->flg; /* save this for nbs_dbrename() */
	if((db_new = nbs_dbcreat(tnam,db_old->flg)) == (NBS_DB *)0) {
		fprintf(stderr,"failed to create %s: %s\n",tnam,nbs_dberror());
		exit(1);
	}

	count = nbs_dbwalkprim(db_old,1,0,-1,copyrec);

	if(nbs_dbclose(db_old)) {
		fprintf(stderr,"failed to close database: %s\n",nbs_dberror());
		exit(-1);
	}
	if(nbs_dbclose(db_new)) {
		fprintf(stderr,"failed to close temporary database: %s\n",nbs_dberror());
		exit(-1);
	}

	if(nbs_dbrename(tnam,dbfilename,dbflags,bflg)) {
		fprintf(stderr,"could not rename database files: %s\n",strerror(errno));
		exit(1);
	}

	if(vflg) {
		printf("retained: %d\n",retains);
		printf("discarded: %d\n",discards);
	}
	exit(0);
}



static	int
tdecode(char *tp)
{
	int	d;
	int	h;
	int	m;

	if(tp == (char *)0)
		return(0);
	if(sscanf(tp,"%d:%d:%d",&d,&h,&m) == 3)
		return((d * 60 * 60 * 24) + (h * 60 * 60) + (m * 60));
	if(sscanf(tp,"%d:%d",&d,&h) == 2)
		return((d * 60 * 60 * 24) + (h * 60 * 60));
	if(sscanf(tp,"%d",&d) == 1)
		return((d * 60 * 60 * 24));
	return(0);
}



/*
copy a record from one db to the other
some duplication from the nbs update rec
code but that's inevitable
*/
static	void
copyrec(char *s,NBS_REC *rp)
{
	NBS_REC	b;
	off_t	rec;
	int	siz;

	/* time out a record - COULD have
	done this by passing a time mask into walkprim()
	but then I wouldn't know how many entries I was
	skipping. */
	if(now - rp->up > tpreen) {
		if(dflg)
			fprintf(stderr,"%s too old (%d) - discarded\n",s,now - rp->up);
		discards++;
		if(vflg)
			fprintf(stderr,"%s\n",s);
		return;
	}

	/* should not be in the db already! */
	if((rec = nbs_dbck(db_new,s,rp->rsiz,(int *)0)) > 0) {
		fprintf(stderr,"WARNING: record \n\t\"%s\"\nis duplicated!! database corrupt?\n",s);
		exit(1);
	}
	if(rec == -1) {
		fprintf(stderr,"primary key check failed! %s\n",nbs_dberror());
		exit(1);
	}

	db_new->ecnt++;
	if(db_new->flg & NBS_DB_FLG_NOREC) {
		rec = b.off = db_new->ecnt;
		b.cnt = -1;
		siz = 0;
	} else {
		rec = nbs_dbnewrec(db_new,rp->rsiz,rp->csiz,&siz);
		b.off = rec;
		b.frst = rp->frst;
		b.up = rp->up;
		b.cnt = rp->cnt;
		b.rsiz = rp->rsiz;
		b.csiz = rp->csiz;
		b.rec = rp->rec;
		b.cmt = rp->cmt;
		if(nbs_dbputrec(db_new,&b,siz)) {
			fprintf(stderr,"record %d put failed! %s\n",(int)rec,nbs_dberror());
			exit(1);
		}
	}

	if(nbs_dbinsprim(db_new,rp->rec,rp->rsiz,b.off,siz)) {
		fprintf(stderr,"primary index put %d failed! %s\n",(int)rec,nbs_dberror());
		exit(1);
	}
	if((db_new->flg & NBS_DB_FLG_NOCOUNT) == 0 && nbs_dbupcounts(db_new,b.cnt,b.off,siz,0)) {
		fprintf(stderr,"record %d put failed! %s\n",(int)rec,nbs_dberror());
		exit(1);
	}
	if((db_new->flg & NBS_DB_FLG_NOUPTIM) == 0 && nbs_dbupup(db_new,rp->frst,rp->up,b.off,siz,0)) {
		fprintf(stderr,"record %d put failed! %s\n",(int)rec,nbs_dberror());
		exit(1);
	}
	if(dflg)
		fprintf(stderr,"copied entry %d count=%d size=%d\n",(int)rec,b.cnt,siz);
	retains++;
}
